% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/low-level.R
\name{ps_interrupt}
\alias{ps_interrupt}
\title{Interrupt a process}
\usage{
ps_interrupt(p = ps_handle(), ctrl_c = TRUE)
}
\arguments{
\item{p}{Process handle or a list of process handles.}

\item{ctrl_c}{On Windows, whether to send 'CTRL+C'. If \code{FALSE}, then
'CTRL+BREAK' is sent. Ignored on non-Windows platforms.}
}
\description{
Sends \code{SIGINT} on POSIX, and 'CTRL+C' or 'CTRL+BREAK' on Windows.
}
\seealso{
Other process handle functions: 
\code{\link{ps_children}()},
\code{\link{ps_cmdline}()},
\code{\link{ps_connections}()},
\code{\link{ps_cpu_times}()},
\code{\link{ps_create_time}()},
\code{\link{ps_cwd}()},
\code{\link{ps_descent}()},
\code{\link{ps_environ}()},
\code{\link{ps_exe}()},
\code{\link{ps_handle}()},
\code{\link{ps_is_running}()},
\code{\link{ps_kill}()},
\code{\link{ps_memory_info}()},
\code{\link{ps_name}()},
\code{\link{ps_num_fds}()},
\code{\link{ps_num_threads}()},
\code{\link{ps_open_files}()},
\code{\link{ps_pid}()},
\code{\link{ps_ppid}()},
\code{\link{ps_resume}()},
\code{\link{ps_send_signal}()},
\code{\link{ps_shared_libs}()},
\code{\link{ps_status}()},
\code{\link{ps_suspend}()},
\code{\link{ps_terminal}()},
\code{\link{ps_terminate}()},
\code{\link{ps_uids}()},
\code{\link{ps_username}()}
}
\concept{process handle functions}
