% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/system.R
\name{ps_boot_time}
\alias{ps_boot_time}
\title{Boot time of the system}
\usage{
ps_boot_time()
}
\value{
A \code{POSIXct} object.
}
\description{
Boot time of the system
}
